% AUTHOR:       Filippo Piccinini (Email: filippo.piccinini85@gmail.com)
%
% DATE:         December 15, 2020
%
% NAME:         START_ForegroundIntensityAnalysis (version 1.00)
%
% DESCRIPTION:  This script have been created to automatically analyse the
%               intensity values of the foreground (i.e. cells) of
%               fluorescent images using external binary masks showing the 
%               pixels belonging to the foreground.
%
% PARAMETERS:
%
% 	pathImms    Absolute path of the folder containing the monochannel
%               8-bit images to be analysed.
%
%   pathMasks   Absolute path of the folder containing the monochannel
%               binary masks showing the foreground. The name of the mask
%               must be perfectly the same of the corresponding monochannel
%               8-bit images to be analysed
%
% OUTPUT:
%
% 	Table       Excel tabble reporting for each image the average and
%               standard deviation of the intensity values of the
%               foreground.

% Copyright (C) 2020 Filippo Piccinini 
% IRCCS Istituto Romagnolo per lo Studio dei Tumori (IRST) "Dino Amadori" 
% Meldola (FC), Italy. All rights reserved.
% Email: filippo.piccinini85@gmail.com
%
% This program is free software; you can redistribute it and/or modify it 
% under the terms of the GNU General Public License version 3 (or higher) 
% as published by the Free Software Foundation. This program is 
% distributed WITHOUT ANY WARRANTY; without even the implied warranty of 
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
% General Public License for more details.

clc, clear, close all


%% Input parameters

pathImms  = 'C:\Users\filippo.piccinini\Desktop\Test01\ImagesToBeAnalysed'; % Write here the full path of the folder containing the monochannel 8-bit images to be analysed.
pathMasks = 'C:\Users\filippo.piccinini\Desktop\Test01\ForegroundMasks'; % Write here the full path of the folder containing the monochannel binary masks showing the foreground of the images to be analysed.


%% Internal parameters

pathOut   = pwd; % Where to automatically save the Excel Table. "pwd" means the folder of this Matlab file 
flag_showStatistics = 1; % 1 = show the pop-up window with the computed values.


%% Correct for common errors

% Slash issues
Slash = filesep;
SlashWrong = '\';
if strcmp(SlashWrong, Slash)
    SlashWrong = '/';
end
% Check pathImms
Pathi = pathImms;
PositionsSlashWrong = strfind(Pathi, SlashWrong);
Pathi(PositionsSlashWrong) = Slash;
if ~strcmp(Pathi(end), Slash)
    Pathi = strcat(Pathi,Slash);
end
pathImms = Pathi;
clear Pathi PositionsSlashWrong
% Check pathMasks
Pathi = pathMasks;
PositionsSlashWrong = strfind(Pathi, SlashWrong);
Pathi(PositionsSlashWrong) = Slash;
if ~strcmp(Pathi(end), Slash)
    Pathi = strcat(Pathi,Slash);
end
pathMasks = Pathi;
clear Pathi PositionsSlashWrong
% Check pathOut
Pathi = pathOut;
PositionsSlashWrong = strfind(Pathi, SlashWrong);
Pathi(PositionsSlashWrong) = Slash;
if ~strcmp(Pathi(end), Slash)
    Pathi = strcat(Pathi,Slash);
end
pathOut = Pathi;
clear Pathi PositionsSlashWrong

%% Load images

dirImms = dir([pathImms '*']);
if isempty(dirImms)
    error('In the selected folder there are not files.')
else
    % To analyse if there are subfolders
    img1_Names = {};
    counter01 = 1;
    for i = 1:length(dirImms)
        StringNameItemi = dirImms(i).name;
        if ~strcmp(StringNameItemi,'.') && ~strcmp(StringNameItemi,'..')
            img1_Names{counter01} = StringNameItemi;
            counter01 = counter01 +1;
        end
    end
end
if length(img1_Names)==0
    error('In the selected folder there are not files.')
end
numImms = counter01-1;


%% Computation

for m = 1:numImms
    
    % Current image to be analysed
    img1_Name = img1_Names{m};
    
    % Image name without extension
    PositionsPoints = strfind(img1_Name, '.');
    PositionLastPoint = PositionsPoints(end);
    StringImg1Type = img1_Name(PositionLastPoint:end);
    StringImg1Name = img1_Name(1:PositionLastPoint-1);
    
    % Image name without extension
    img1  = imread([pathImms img1_Name]);
    mask1 = imread([pathMasks img1_Name]);
    
    % Pre-processing
    mask1(mask1>1) = 1;
    
    % Processing
    img2 = img1.*mask1;
    img1_mean = mean(img2(mask1==1));
    img1_std = std(double(img2(mask1==1)));
    
    % Table preparation
    TableOutput.ID{m,1} = m;
    TableOutput.Name{m,1} = StringImg1Name;
    TableOutput.IntensityMean{m,1} = img1_mean;
    TableOutput.IntensityStd{m,1} = img1_std;
    
    clear img1_std img1_mean StringImg1Name StringImg1Type PositionLastPoint PositionsPointsmask1 img1 img2 img1_Name

end


%% Save output table

if numImms>1
    function_ExportExcelTable(TableOutput, pathOut, 'OutputTable');
    if flag_showStatistics == 1
        function_ShowDataExtracted(TableOutput, 'OutputTable')
    end
end